@@ -337,9 +337,16 @@ handle_dwd <- function(action,
     
     myPoint <- c(longitude, latitude)
     
-    station_in_period[, "Distance"] <- round(sp::spDistsN1(
-      as.matrix(station_in_period[, c("Longitude", "Latitude")]), myPoint,
-      longlat = TRUE), 2) 
+    lat_rad <- latitude*pi/180
+    lon_rad <- longitude*pi/180
+    lat_rad_stat <- station_in_period$Latitude*pi/180
+    lon_rad_stat <- station_in_period$Longitude*pi/180
+    
+    station_in_period[, "Distance"] <-
+      round(6378.388 * acos(sin(lat_rad) * sin(lat_rad_stat) +
+                              cos(lat_rad) * cos(lat_rad_stat) *
+                              cos(lon_rad_stat - lon_rad)), 
+            2) 
     
     # Order the station according to the distance to the point
     
